@@ -33,6 +33,7 @@ import asyncio
 import shutil
 import websockets
 from uuid import uuid4
+from contextlib import asynccontextmanager
 from bokeh.models import Button, TextInput, Div, Range1d, LinearAxis, CustomJS, Spacer
 from bokeh.plotting import ColumnDataSource, figure, show
 from bokeh.layouts import column, row, Spacer
@@ -41,7 +42,7 @@ from ..utils import resource_manager, reset_resource_manager
 
 try:
     ## gclean version number needed for proper interactive clean behavior
-    from casatasks.private.imagerhelpers._gclean import _GCV001
+    from casatasks.private.imagerhelpers._gclean import _GCV003
     from casatasks.private.imagerhelpers._gclean import gclean as _gclean
 except:
     try:
@@ -49,7 +50,7 @@ except:
         ### enable this warning when casa6 a usable _gclean.py (i.e. compatibility is not the default)
         ###
         #print('warning: using tclean compatibility layer...')
-        from ..private.compatibility.casatasks.private.imagerhelpers._gclean import _GCV001
+        from ..private.compatibility.casatasks.private.imagerhelpers._gclean import _GCV003
         from ..private.compatibility.casatasks.private.imagerhelpers._gclean import gclean as _gclean
     except:
         _gclean = None
@@ -152,13 +153,9 @@ class InteractiveClean:
 
     '''
     def __stop( self ):
-        resource_manager( ).stop_asyncio_loop()
-        if self._control_server is not None and self._control_server.ws_server.is_serving( ):
-            resource_manager( ).stop_asyncio_loop()
-        if self._converge_server is not None and self._converge_server.ws_server.is_serving( ):
-            resource_manager( ).stop_asyncio_loop()
+        self.__result_future.set_result(self.__retrieve_result( ))
 
-    def _abort_handler( self, loop, err ):
+    def _abort_handler( self, err ):
         self._error_result = err
         self.__stop( )
 
@@ -169,6 +166,11 @@ class InteractiveClean:
             reset_resource_manager( )
             self._clean.reset( )
 
+        ###
+        ### reset asyncio result future
+        ###
+        self.__result_future = None
+
         ###
         ### used by data pipe (websocket) initialization function
         ###
@@ -200,6 +202,11 @@ class InteractiveClean:
         if deconvolver == 'mtmfs':
             raise RuntimeError("deconvolver task does not support 'mtmf' deconvolver")
 
+        ###
+        ### the asyncio future that is used to transmit the result from interactive clean
+        ###
+        self.__result_future = None
+
         ###
         ### This is used to tell whether the websockets have been initialized, but also to
         ### indicate if __call__ is being called multiple times to allow for resetting Bokeh
@@ -253,6 +260,8 @@ class InteractiveClean:
         ###
         self._status = { }
         err, stopcode, self._convergence_data = next(self._clean)
+        if stopcode is None and err:
+            raise RuntimeError(err)
         if stopcode > 1 and stopcode < 9: # 1: iteration limit hit, 9: major cycle limit hit
             self._clean.finalize()
         if len(self._convergence_data.keys()) == 0:
@@ -446,9 +455,15 @@ class InteractiveClean:
                                                    enable(false)
                                                    state.stopped = false
                                                    update_status( msg.error )
+                                                   if ( 'cmd' in msg ) {
+                                                       log.text = log.text + msg.cmd
+                                                   }
                                                } else if ( msg.result === 'no-action' ) {
                                                    update_status( 'nothing done' )
                                                    enable( false )
+                                                   if ( 'cmd' in msg ) {
+                                                       log.text = log.text + msg.cmd
+                                                   }
                                                } else if ( msg.result === 'update' ) {
                                                    if ( 'cmd' in msg ) {
                                                        log.text = log.text + msg.cmd
@@ -569,10 +584,11 @@ class InteractiveClean:
                 #    self._clean.finalize()
 
                     # self._cube.update_image(self._clean.finalize()['image']) # TODO show the restored image
-                if len(self._convergence_data) == 0 and stopcode == 7:
-                    return dict( result='error', stopcode=stopcode, cmd=f"<p>mask error encountered (stopcode {stopcode})</p>", convergence=None, error=err )
+                if len(self._convergence_data) == 0 and stopcode == 7 or err:
+                    return dict( result='error', stopcode=stopcode, cmd=''.join([ f'<p style="width:790px">{cmd}</p>' for cmd in self._clean.cmds( )[-2:] ]),
+                                 convergence=None, error=err )
                 if len(self._convergence_data) == 0:
-                    return dict( result='no-action', stopcode=stopcode, cmd=f'<p style="width:790px">no operation</p>',
+                    return dict( result='no-action', stopcode=stopcode, cmd=''.join([ f'<p style="width:790px">{cmd}</p>' for cmd in self._clean.cmds( )[-2:] ]),
                                  convergence=None, iterdone=0, error=err )
                 if len(self._convergence_data) * len(self._convergence_data[0]) > self._threshold_chan or \
                    len(self._convergence_data[0][0]['iterations']) > self._threshold_iterations:
@@ -804,7 +820,8 @@ class InteractiveClean:
                                           row ( Div( text="<div><b>status:</b></div>" ), self._status['stopcode'] ),
                                           self._cube.statistics( sizing_mode = "stretch_height" ),
                                       ),
-                                      column( self._fig['image'],
+                                      column( self._cube.channel_label( ),
+                                              self._fig['image'],
                                               row( Spacer(height=self._control['help'].height, sizing_mode="scale_width"),
                                                    self._cube.palette( ),
                                                    mask_clean_notclean_pick,
@@ -872,23 +889,8 @@ class InteractiveClean:
         self._cube.connect( )
         show(self._fig['layout'])
 
-    def _asyncio_loop( self ):
-        async def async_loop( f1, f2, f3 ):
-            return await asyncio.gather( f1, f2, f3 )
-
-        self._control_server = websockets.serve( self._pipe['control'].process_messages, self._pipe['control'].address[0], self._pipe['control'].address[1] )
-        self._converge_server = websockets.serve( self._pipe['converge'].process_messages, self._pipe['converge'].address[0], self._pipe['converge'].address[1] )
-        resource_manager( ).reg_webserver(self._control_server.ws_server)
-        resource_manager( ).reg_webserver(self._converge_server.ws_server)
-        return async_loop( self._control_server, self._converge_server, self._cube.loop( ) )
-
-    def __call__( self, loop=asyncio.new_event_loop( ) ):
-        '''Display GUI using the event loop specified by ``loop``.
-
-        Parameters
-        ----------
-        loop: event loop object
-            defaults to standard asyncio event loop.
+    def __call__( self ):
+        '''Display GUI and process events until the user stops the application.
 
         Example:
             Create ``iclean`` object and display::
@@ -898,30 +900,50 @@ class InteractiveClean:
                                cell='12.0arcsec', specmode='cube',
                                interpolation='nearest', ... )( ) )
         '''
-        ###
-        ### new event loops must be set in asyncio to become active
-        ###
-        if loop != asyncio.get_event_loop( ):
-            asyncio.set_event_loop(loop)
-
-        self.__reset( )
-
-        try:
-            loop.run_until_complete(self.show( ))
-            loop.run_forever( )
-        except KeyboardInterrupt:
-            print('\nInterrupt received, stopping GUI...')
+        async def _run_( ):
+            async with self.serve( ) as s:
+                await s[0]
 
+        asyncio.run(_run_( ))
         return self.result( )
 
-    def show( self ):
-        '''Get the InteractiveClean event loop to use for running the interactive clean GUI
-        as part of an external event loop.
-        '''
-        self._launch_gui( )
-        return self._asyncio_loop( )
+    @asynccontextmanager
+    async def serve( self ):
+        '''This function is intended for developers who would like to embed interactive
+        clean as a part of a larger GUI. This embedded use of interactive clean is not
+        currently supported and would require the addition of parameters to this function
+        as well as changes to the interactive clean implementation. However, this function
+        does expose the ``asyncio.Future`` that is used to signal completion of the
+        interactive cleaning operation, and it provides the coroutines which must be
+        managed by asyncio to make the interactive clean GUI responsive.
 
-    def result( self ):
+        Example:
+            Create ``iclean`` object, process events and retrieve result::
+
+                ic = iclean( vis='refim_point_withline.ms', imagename='test', imsize=512,
+                             cell='12.0arcsec', specmode='cube', interpolation='nearest', ... )
+                async def process_events( ):
+                    async with ic.serve( ) as state:
+                        await state[0]
+
+                asyncio.run(process_events( ))
+                print( "Result:", ic.result( ) )
+
+
+        Returns
+        -------
+        (asyncio.Future, dictionary of coroutines)
+        '''
+        self.__reset( )
+        self._launch_gui( )
+
+        async with websockets.serve( self._pipe['control'].process_messages, self._pipe['control'].address[0], self._pipe['control'].address[1] ) as ctrl, \
+                   websockets.serve( self._pipe['converge'].process_messages, self._pipe['converge'].address[0], self._pipe['converge'].address[1] ) as conv, \
+                   self._cube.serve( self.__stop ) as cube:
+            self.__result_future = asyncio.Future( )
+            yield ( self.__result_future, { 'ctrl': ctrl, 'conv': conv, 'cube': cube } )
+
+    def __retrieve_result( self ):
         '''If InteractiveClean had a return value, it would be filled in as part of the
         GUI dialog between Python and JavaScript and this function would return it'''
         if isinstance(self._error_result,Exception):
@@ -930,6 +952,14 @@ class InteractiveClean:
             return self._error_result
         return self._cube.result( )
 
+    def result( self ):
+        '''If InteractiveClean had a return value, it would be filled in as part of the
+        GUI dialog between Python and JavaScript and this function would return it'''
+        if self.__result_future is None:
+            raise RuntimeError( 'no interactive clean result is available' )
+        self._clean.restore( )
+        return self.__result_future.result( )
+
     def masks( self ):
         '''Retrieves the masks which were used with interactive clean.
 
