@@ -31,6 +31,8 @@ from functools import reduce
 import copy
 
 _GCV001 = True
+_GCV002 = True
+_GCV003 = True
 
 # from casatasks.private.imagerhelpers._gclean import gclean
 class gclean:
@@ -281,7 +283,9 @@ class gclean:
                                          self._convergence_result[1],
                                          self._convergence_result[2] )
             raise StopIteration
-        if self._niter < 1:
+        #                                                                             ensure that at least the initial tclean run
+        #                      vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv------------>>> is done to produce an initial dirty image
+        if self._niter < 1 and self._convergence_result[2] is not None:
             self._convergence_result = ( f'nothing to run, niter == {self._niter}',
                                          self._convergence_result[1],
                                          self._convergence_result[2] )
@@ -303,7 +307,7 @@ class gclean:
                                            restoringbeam=self._restoringbeam, pbcor=self._pbcor, nterms=self._nterms, field=self._field, spw=self._spw,
                                            timerange=self._timerange, uvrange=self._uvrange, antenna=self._antenna, scan=self._scan,
                                            observation=self._observation, intent=self._intent, datacolumn=self._datacolumn, weighting=self._weighting,
-                                           robust=self._robust, npixels=self._npixels, interactive=0, niter=1, gain=0.000001, calcres=True,
+                                           robust=self._robust, npixels=self._npixels, interactive=False, niter=1, gain=0.000001, calcres=True,
                                            restoration=False, parallel=self._parallel )
                 self._deconvolve( imagename=self._imagename, niter=0, usemask=self._usemask, restoration=False, deconvolver=self._deconvolver )
             else:
@@ -316,7 +320,7 @@ class gclean:
                                            deconvolver=self._deconvolver, cyclefactor=self._cyclefactor, scales=self._scales, restoringbeam=self._restoringbeam,
                                            pbcor=self._pbcor, nterms=self._nterms, field=self._field, spw=self._spw, timerange=self._timerange,
                                            uvrange=self._uvrange, antenna=self._antenna, scan=self._scan, observation=self._observation, intent=self._intent,
-                                           datacolumn=self._datacolumn, weighting=self._weighting, robust=self._robust, npixels=self._npixels, interactive=0,
+                                           datacolumn=self._datacolumn, weighting=self._weighting, robust=self._robust, npixels=self._npixels, interactive=False,
                                            niter=self._niter, restart=True, calcpsf=False, calcres=False, restoration=False, threshold=self._threshold,
                                            nsigma=self._nsigma, cycleniter=self._cycleniter, nmajor=self._nmajor, gain=self._gain,
                                            sidelobethreshold=self._sidelobethreshold, noisethreshold=self._noisethreshold,
@@ -326,10 +330,15 @@ class gclean:
                                            minpsffraction=0, parallel=self._parallel )
                 self._deconvolve( imagename=self._imagename, niter=0, usemask=self._usemask, restoration=False, deconvolver=self._deconvolver )
 
-            new_summaryminor_rec = gclean.__filter_convergence(tclean_ret['summaryminor'])
-            self._convergence_result = ( None,
-                                         tclean_ret['stopcode'] if 'stopcode' in tclean_ret else 0,
-                                         gclean.__update_convergence(self._convergence_result[2],new_summaryminor_rec) )
+            if len(tclean_ret) > 0 and 'summaryminor' in tclean_ret and sum(map(len,tclean_ret['summaryminor'].values())) > 0:
+                new_summaryminor_rec = gclean.__filter_convergence(tclean_ret['summaryminor'])
+                self._convergence_result = ( None,
+                                             tclean_ret['stopcode'] if 'stopcode' in tclean_ret else 0,
+                                             gclean.__update_convergence(self._convergence_result[2],new_summaryminor_rec) )
+            else:
+                self._convergence_result = ( f'tclean returned an empty result',
+                                             self._convergence_result[1],
+                                             self._convergence_result[2] )
             return self._convergence_result
 
     def __reflect_stop( self ):
@@ -340,6 +349,8 @@ class gclean:
             raise StopAsyncIteration
 
     async def __anext__( self ):
+        ### asyncio.run cannot be used here because this is called
+        ### within an asyncio loop...
         loop = asyncio.get_event_loop( )
         result = await loop.run_in_executor( None, self.__reflect_stop )
         return result
@@ -358,30 +369,26 @@ class gclean:
                                      self._convergence_result[1],
                                      self._convergence_result[2] )
 
-    def finalize(self):
-        """ Runs the restoration step, prevents any future cleaning, and returns a path to the restored image. """
-        if not self._finalized:
-            self._finalized = True
-            tclean_ret = self._tclean( vis=self._vis, imagename=self._imagename, imsize=self._imsize, cell=self._cell,
-                                       phasecenter=self._phasecenter, stokes=self._stokes, specmode=self._specmode,
-                                       reffreq=self._reffreq, gridder=self._gridder, wprojplanes=self._wprojplanes, mosweight=self._mosweight,
-                                       psterm=self._psterm, wbawp=self._wbawp, conjbeams=self._conjbeams, usepointing=self._usepointing,
-                                       interpolation=self._interpolation, perchanweightdensity=self._perchanweightdensity, nchan=self._nchan,
-                                       start=self._start, width=self._width, outframe=self._outframe, pointingoffsetsigdev=self._pointingoffsetsigdev,
-                                       pblimit=self._pblimit, deconvolver=self._deconvolver, cyclefactor=self._cyclefactor, scales=self._scales,
-                                       restoringbeam=self._restoringbeam, pbcor=self._pbcor, nterms=self._nterms, field=self._field, spw=self._spw,
-                                       timerange=self._timerange, uvrange=self._uvrange, antenna=self._antenna, scan=self._scan,
-                                       observation=self._observation, intent=self._intent, datacolumn=self._datacolumn, weighting=self._weighting,
-                                       robust=self._robust, npixels=self._npixels, gain=self._gain, sidelobethreshold=self._sidelobethreshold,
-                                       noisethreshold=self._noisethreshold, lownoisethreshold=self._lownoisethreshold,
-                                       negativethreshold=self._negativethreshold, minbeamfrac=self._minbeamfrac, growiterations=self._growiterations,
-                                       dogrowprune=self._dogrowprune, minpercentchange=self._minpercentchange, fastnoise=self._fastnoise,
-                                       savemodel=self._savemodel, nsigma=self._nsigma, interactive=0,
-                                       niter=0, restart=True, calcpsf=False, calcres=False, restoration=True,
-                                       parallel=self._parallel )
-        return {
-            "image": f"{self._imagename}.image"
-        }
+    def restore(self):
+        """ Restores the final image, and returns a path to the restored image. """
+        tclean_ret = self._tclean( vis=self._vis, imagename=self._imagename, imsize=self._imsize, cell=self._cell,
+                                   phasecenter=self._phasecenter, stokes=self._stokes, specmode=self._specmode,
+                                   reffreq=self._reffreq, gridder=self._gridder, wprojplanes=self._wprojplanes, mosweight=self._mosweight,
+                                   psterm=self._psterm, wbawp=self._wbawp, conjbeams=self._conjbeams, usepointing=self._usepointing,
+                                   interpolation=self._interpolation, perchanweightdensity=self._perchanweightdensity, nchan=self._nchan,
+                                   start=self._start, width=self._width, outframe=self._outframe, pointingoffsetsigdev=self._pointingoffsetsigdev,
+                                   pblimit=self._pblimit, deconvolver=self._deconvolver, cyclefactor=self._cyclefactor, scales=self._scales,
+                                   restoringbeam=self._restoringbeam, pbcor=self._pbcor, nterms=self._nterms, field=self._field, spw=self._spw,
+                                   timerange=self._timerange, uvrange=self._uvrange, antenna=self._antenna, scan=self._scan,
+                                   observation=self._observation, intent=self._intent, datacolumn=self._datacolumn, weighting=self._weighting,
+                                   robust=self._robust, npixels=self._npixels, gain=self._gain, sidelobethreshold=self._sidelobethreshold,
+                                   noisethreshold=self._noisethreshold, lownoisethreshold=self._lownoisethreshold,
+                                   negativethreshold=self._negativethreshold, minbeamfrac=self._minbeamfrac, growiterations=self._growiterations,
+                                   dogrowprune=self._dogrowprune, minpercentchange=self._minpercentchange, fastnoise=self._fastnoise,
+                                   savemodel=self._savemodel, nsigma=self._nsigma, interactive=False,
+                                   niter=0, restart=True, calcpsf=False, calcres=False, restoration=True,
+                                   parallel=self._parallel )
+        return { "image": f"{self._imagename}.image" }
 
     def has_next(self):
         return not self._finalized
